@@ -10,15 +10,14 @@
 
 /*
 ** \file procutil.h
-** \version $Id: procutil.h 2571 2008-05-10 04:23:18Z edmanm $
+** \version $Id: procutil.h 3751 2009-05-01 03:26:38Z edmanm $
 ** \brief Process information and pidfile functions
 */
 
 #ifndef _PROCUTIL_H
 #define _PROCUTIL_H
 
-#include <QString>
-
+#include <QtGlobal>
 #if defined(Q_OS_WIN)
 #include "win32.h"
 #else
@@ -28,6 +27,9 @@
 #include <errno.h>
 #endif
 
+#include <QHash>
+
+class QString;
 
 /** Returns the PID of the current process. */
 qint64 get_pid();
@@ -36,11 +38,19 @@ qint64 get_pid();
 bool is_process_running(qint64 pid);
 
 /** Writes the given file to disk containing the current process's PID. */
-bool write_pidfile(QString pidfile, QString *errmsg = 0);
+bool write_pidfile(const QString &pidfile, QString *errmsg = 0);
 
 /** Reads the giiven pidfile and returns the value in it. If the file does not
  * exist, -1 is returned. */
-qint64 read_pidfile(QString pidfile, QString *errmsg = 0);
+qint64 read_pidfile(const QString &pidfile, QString *errmsg = 0);
+
+/** Return a list of all currently running PIDs and their associated process
+ * names. */
+QHash<qint64, QString> process_list();
+
+/** Attempt to kill process <b>pid</b>. Return true if the specified process
+ * was successfully terminated. Otherwise, return false. */
+bool process_kill(qint64 pid);
 
 #endif
 
