@@ -10,17 +10,19 @@
 
 /*
 ** \file win32.cpp
-** \version $Id: win32.cpp 3527 2009-02-16 23:28:41Z sjmurdoch $ 
+** \version $Id: win32.cpp 4294 2010-05-18 03:58:45Z edmanm $ 
 ** \brief Win32-specific functions
 */
 
 #include "win32.h"
-#include <tlhelp32.h>
-#include <shlobj.h>
+
 #include <QDir>
 #include <QLibrary>
 #include <QtDebug>
 
+#include <tlhelp32.h>
+#include <shlobj.h>
+
 #if defined(UNICODE)
 /* Force the ascii verisons of these functions, so we can run on Win98. We
  * don't pass any Unicode strings to these functions anyway. */
@@ -47,6 +49,12 @@ win32_get_folder_location(int folder, QString defaultPath)
   IMalloc *m;
   HRESULT result;
 
+  /* XXX: It would probably be a good idea to replace this function with simple
+   *      calls to QDesktopServices::storageLocation(). Note that it returns the
+   *      "Local Settings" application data directory though, so our installers
+   *      would need to be updated as well.
+   */
+
   /* Find the location of %PROGRAMFILES% */
   if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, folder, &idl))) {
     /* Get the path from the IDL */
@@ -56,8 +64,11 @@ win32_get_folder_location(int folder, QString defaultPath)
       m->Release();
     }
     if (SUCCEEDED(result)) {
-      QT_WA(return QString::fromUtf16((const ushort *)path);,
-            return QString::fromLocal8Bit((char *)path);)
+#if defined(UNICODE)
+      return QString::fromUtf16(static_cast<const ushort *>(path));
+#else
+      return QString::fromLocal8Bit(static_cast<const char *>(path));
+#endif
     }
   }
   return defaultPath;
@@ -153,7 +164,8 @@ win32_registry_remove_key(QString keyLocation, QString keyName)
 /**
  * Callback for EnumThreadWindows which sends the WM_QUIT message
  */
-BOOL CALLBACK quitWindowCallback(HWND hwnd, LPARAM targetPID)
+BOOL CALLBACK 
+quitWindowCallback(HWND hwnd, LPARAM targetPID)
 {
   DWORD hwndPID = 0;
 
@@ -162,6 +174,7 @@ BOOL CALLBACK quitWindowCallback(HWND hwnd, LPARAM targetPID)
   GetWindowThreadProcessId(hwnd, &hwndPID);
   if (hwndPID == (DWORD)targetPID)
     PostMessage(hwnd, WM_QUIT, 0, (LPARAM)NULL);
+  return TRUE;
 }
 
 /**
